#include "app.h"
#include "FileMenu.h"

char LoopQuit=0;
static FILE *DebugFile=NULL;
int LoopMode=0;

int LoopInit()
{
  int ret=0;

  // Init Direct3D:
  ret=DirectInit(); if (ret) return 1;
  InputInit();

  // Init DirectSound:
  DSoundSeg=735;
  if (MainHeight==576) DSoundSeg=882; // PAL 50hz mode
  DSoundInit();

  ret=EmuInit(); if (ret) return 1;
  FileMenu.init();

  return 0;
}

void LoopExit()
{
  FileMenu.exit();
  EmuExit();
  DSoundExit(); DSoundSeg=0;
  InputExit();
  DirectExit();

  if (DebugFile) fclose(DebugFile);
  DebugFile=NULL;
}

// ----------------------------------------------------------------

static int DoGame()
{
  EmuFrame();

  if (Inp.held[7]==1) LoopMode=2; // Right thumb = Toggle Menu

  return 0;
}
// ----------------------------------------------------------------

static int MenuUpdate()
{
  int delta=0;

  if (Inp.repeat[0]) delta-=0x100;
  if (Inp.repeat[1]) delta+=0x100;

  if (Inp.button[14]>30) delta-=Inp.button[14]-30;
  if (Inp.button[15]>30) delta+=Inp.button[15]-30;

  if (delta) FileMenu.scroll(delta);

  if (Inp.held[8]==1 || Inp.held[10]==1 || Inp.held[4]==1) // A, X or Start
  {
    RomFree();
    FileMenu.getFilePath(RomName);
    RomLoad();
    LoopMode=8; // Go to game
  }

  if (Inp.held[7]==1) LoopMode=8; // Right thumb = Toggle Menu

  return 0;
}

static int MenuRender()
{
  WCHAR text[80]={0};
  wsprintfW(text,L"%.40S v%x.%.3x",AppName,PicoVer>>12,PicoVer&0xfff);
  FontSetColour(0x60c0ff);
  FontText(text,64,48);

  FileMenu.render();

  return 0;
}

// ----------------------------------------------------------------

static int ModeUpdate()
{
  if (Inp.held[14] && Inp.held[15] && Inp.held[12]==1) LoopQuit=1; // L+R+black to quit:
  if (Inp.button[4]>30 && Inp.button[5]>30) LoopQuit=1; // Start and back to quit

  if (LoopMode==8) { DoGame(); return 0; }

  if (DSoundNext) memset(DSoundNext,0,DSoundSeg<<2);

  if (LoopMode==2) { FileMenu.scan(); LoopMode++; return 0; }
  if (LoopMode==3) { MenuUpdate(); return 0; }
  if (LoopMode==4) { LightCalUpdate(); return 0; }

  LoopMode=2; // Unknown mode, go to rom menu
  return 0;
}


static int ModeRender()
{
  DirectScreen();
  if (LoopMode==3) MenuRender();
  if (LoopMode==4) LightCalRender();

  return 0;
}

static int UpdateSound()
{
  while (DSoundUpdate()) { Sleep(1); }
  while (DSoundUpdate()==0) { }
  return 0;
}

int LoopCode()
{

  // Main loop:
  while (!LoopQuit)
  {
    InputUpdate();

    DirectClear(0);
    ModeUpdate();
    ModeRender();
    DirectPresent();
    UpdateSound();
  }

  return 0;
}

// -------------------------------------------------------------------------------------

extern "C" int dprintf(char *format, ...)
{
  char *name=NULL;
  va_list val=NULL;

#ifdef _XBOX
  name="d:\\zout.txt";
#else
  name="zout.txt";
#endif

  if (DebugFile==NULL) DebugFile=fopen(name,"wt");
  if (DebugFile==NULL) return 1;

  va_start(val,format);
  vfprintf(DebugFile,format,val);
  fflush(DebugFile);

  va_end(val);
  return 0;
}
